@@ -1,6 +1,6 @@
 /****************************************************************/
-/*      Stanton SCS.3d MIDI controller script v1.60             */
-/*          Copyright (C) 2009-2010, Sean M. Pappalardo         */
+/*      Stanton SCS.3d MIDI controller script v1.70             */
+/*          Copyright (C) 2009-2011, Sean M. Pappalardo         */
 /*      but feel free to tweak this to your heart's content!    */
 /*      For Mixxx version 1.9.x                                 */
 /****************************************************************/
@@ -21,6 +21,9 @@ StantonSCS3d.globalMode = false;        // Stay in the current mode on deck chan
 StantonSCS3d.singleDeck = false;        // When using more than one controller, set to true to avoid easy deck changes.
                                         //  Toggle with DECK + SYNC.
 StantonSCS3d.deckChangeWait = 1000;     // Time in milliseconds to hold the DECK button down to avoid changing decks (multi deck mode)
+StantonSCS3d.pitchAdjustment = 3;       // Pitch slider coarseness (1 = coarse, 3 = normal, 5 = fine)
+StantonSCS3d.finePitchAdjustment = 3;   // Fine-mode pitch slider coarseness (1 = coarser, 3 = normal, 5 = finer)
+StantonSCS3d.finePitchDefault = false;  // Swap normal and fine pitch modes
 
 // ----------   Other global variables    ----------
 StantonSCS3d.debug = false;  // Enable/disable debugging messages to the console
@@ -60,7 +63,7 @@ StantonSCS3d.hotCues = {    1:{ 0x48: 1, 0x4A: 2, 0x4C: 3, 0x4E: 4, 0x4F: 5, 0x5
                             2:{ 0x48:13, 0x4A:14, 0x4C:15, 0x4E:16, 0x4F:17, 0x51:18,
                                 0x53:19, 0x55:20, 0x56:21, 0x58:22, 0x5A:23, 0x5C:24 },
                             3:{ 0x48:25, 0x4A:26, 0x4C:27, 0x4E:28, 0x4F:29, 0x51:30,
-                                0x53:31, 0x55:-1, 0x56:-1, 0x58:-1, 0x5A:-1, 0x5C:-1 } };
+                                0x53:31, 0x55:32, 0x56:33, 0x58:34, 0x5A:35, 0x5C:36 } };
 StantonSCS3d.triggerS4 = 0xFF;
 
 // Signals to (dis)connect by mode: Group, Key, Function name
@@ -137,13 +140,23 @@ StantonSCS3d.modeSignals = {"fx":[    ["[Flanger]", "lfoDepth", "StantonSCS3d.FX
                                       ["CurrentChannel", "hotcue_29_enabled", "StantonSCS3d.BsELED"],
                                       ["CurrentChannel", "hotcue_30_enabled", "StantonSCS3d.BsFLED"],
                                       ["CurrentChannel", "hotcue_31_enabled", "StantonSCS3d.BsGLED"],
+                                      ["CurrentChannel", "hotcue_32_enabled", "StantonSCS3d.BsHLED"],
+                                      ["CurrentChannel", "hotcue_33_enabled", "StantonSCS3d.BsILED"],
+                                      ["CurrentChannel", "hotcue_34_enabled", "StantonSCS3d.BsJLED"],
+                                      ["CurrentChannel", "hotcue_35_enabled", "StantonSCS3d.BsKLED"],
+                                      ["CurrentChannel", "hotcue_36_enabled", "StantonSCS3d.BsLLED"],
                                       ["CurrentChannel", "hotcue_25_activate", "StantonSCS3d.BsAaLED"],
                                       ["CurrentChannel", "hotcue_26_activate", "StantonSCS3d.BsBaLED"],
                                       ["CurrentChannel", "hotcue_27_activate", "StantonSCS3d.BsCaLED"],
                                       ["CurrentChannel", "hotcue_28_activate", "StantonSCS3d.BsDaLED"],
                                       ["CurrentChannel", "hotcue_29_activate", "StantonSCS3d.BsEaLED"],
                                       ["CurrentChannel", "hotcue_30_activate", "StantonSCS3d.BsFaLED"],
-                                      ["CurrentChannel", "hotcue_31_activate", "StantonSCS3d.BsGaLED"] ],
+                                      ["CurrentChannel", "hotcue_31_activate", "StantonSCS3d.BsGaLED"],
+                                      ["CurrentChannel", "hotcue_32_activate", "StantonSCS3d.BsHaLED"],
+                                      ["CurrentChannel", "hotcue_33_activate", "StantonSCS3d.BsIaLED"],
+                                      ["CurrentChannel", "hotcue_34_activate", "StantonSCS3d.BsJaLED"],
+                                      ["CurrentChannel", "hotcue_35_activate", "StantonSCS3d.BsKaLED"],
+                                      ["CurrentChannel", "hotcue_36_activate", "StantonSCS3d.BsLaLED"] ],
                             "vinyl":[ ["CurrentChannel", "pfl", "StantonSCS3d.B11LED"],
                                       ["CurrentChannel", "VuMeter", "StantonSCS3d.VUMeterLEDs"] ],
                             "vinyl2":[["CurrentChannel", "pfl", "StantonSCS3d.B11LED"],
@@ -386,11 +399,20 @@ StantonSCS3d.pitch = function (channel, control, value) {   // Lower the sensiti
     // If in DECK mode, ignore this.
     if (StantonSCS3d.mode_store["[Channel"+StantonSCS3d.deck+"]"]=="deck") return;
     
+    // Ignore if doing pitch bend
+    if (engine.getValue("[Channel"+StantonSCS3d.deck+"]","rate_temp_up")!=0
+        || engine.getValue("[Channel"+StantonSCS3d.deck+"]","rate_temp_down")!=0) return;
+    
     var currentValue = engine.getValue("[Channel"+StantonSCS3d.deck+"]","rate");
     var newValue;
-    if (StantonSCS3d.modifier[StantonSCS3d.mode_store["[Channel"+StantonSCS3d.deck+"]"]]==1)
-        newValue = currentValue+(value-64)/1024;    // Fine pitch adjust
-    else newValue = currentValue+(value-64)/256;
+    var modDeck = StantonSCS3d.modifier[StantonSCS3d.mode_store["[Channel"+StantonSCS3d.deck+"]"]]==1;
+    var swap = StantonSCS3d.finePitchDefault;
+    if ((modDeck && !swap) || (!modDeck && swap)) {
+        // Fine pitch adjust
+        var pitchRange = engine.getValue("[Channel"+StantonSCS3d.deck+"]","rateRange");
+        newValue = currentValue+(value-64)/(20000*StantonSCS3d.finePitchAdjustment*pitchRange);
+    }
+    else newValue = currentValue+(value-64)/(86*StantonSCS3d.pitchAdjustment);
     if (newValue<-1) newValue=-1.0;
     if (newValue>1) newValue=1.0;
     engine.setValue("[Channel"+StantonSCS3d.deck+"]","rate",newValue);
@@ -407,6 +429,10 @@ StantonSCS3d.pitchAbsolute = function (channel, control, value) {
     // Disable if doing fine adjustments (holding down the current mode button)
     if (StantonSCS3d.modifier[StantonSCS3d.mode_store["[Channel"+StantonSCS3d.deck+"]"]]==1) return;
     
+    // Ignore if we're already bending
+    if (engine.getValue("[Channel"+StantonSCS3d.deck+"]","rate_temp_up")!=0
+        || engine.getValue("[Channel"+StantonSCS3d.deck+"]","rate_temp_down")!=0) return;
+    
     // --- Pitch bending at the edges of the slider ---
     if (StantonSCS3d.state["pitchAbs"]==0) StantonSCS3d.state["pitchAbs"]=value;    // Log the initial value
     
@@ -472,12 +498,8 @@ StantonSCS3d.playButton = function (channel, control, value, status) {
             return
         }
         StantonSCS3d.modifier["play"]=1;
-        if (StantonSCS3d.modifier["cue"]==1) engine.setValue("[Channel"+StantonSCS3d.deck+"]","play",1);
-        else {
-            var currentlyPlaying = engine.getValue("[Channel"+StantonSCS3d.deck+"]","play");
-            if (currentlyPlaying && engine.getValue("[Channel"+StantonSCS3d.deck+"]","cue_default")==1) engine.setValue("[Channel"+StantonSCS3d.deck+"]","cue_default",0);
-            engine.setValue("[Channel"+StantonSCS3d.deck+"]","play", !currentlyPlaying);
-        }
+        var currentlyPlaying = engine.getValue("[Channel"+StantonSCS3d.deck+"]","play");
+        engine.setValue("[Channel"+StantonSCS3d.deck+"]","play", !currentlyPlaying);
         return;
     }
     StantonSCS3d.modifier["play"]=0;
@@ -501,8 +523,7 @@ StantonSCS3d.cueButton = function (channel, control, value, status) {
         StantonSCS3d.modifier["cue"]=1;   // Set button modifier flag
         return;
     }
-    if (StantonSCS3d.modifier["play"]==0 && !StantonSCS3d.modifier["vinyl2"])
-        engine.setValue("[Channel"+StantonSCS3d.deck+"]","cue_default",0);
+    engine.setValue("[Channel"+StantonSCS3d.deck+"]","cue_default",0);
     StantonSCS3d.modifier["cue"]=0;   // Clear button modifier flag
 }
 
@@ -656,7 +677,8 @@ StantonSCS3d.B12 = function (channel, control, value, status) {
                 break;
         default:
                 if ((status & 0xF0) == 0x90) {    // If button down
-                    var currentRange = engine.getValue("[Channel"+StantonSCS3d.deck+"]","rateRange");
+                    // Round to two decimal places to avoid double-precision comparison problems
+                    var currentRange = Math.round(engine.getValue("[Channel"+StantonSCS3d.deck+"]","rateRange")*100)/100;
                     switch (true) {
                         case (currentRange<=StantonSCS3d.pitchRanges[0]):
                                 engine.setValue("[Channel"+StantonSCS3d.deck+"]","rateRange",StantonSCS3d.pitchRanges[1]);
@@ -780,8 +802,7 @@ StantonSCS3d.modeButton = function (channel, control, status, modeName) {
     }
     StantonSCS3d.connectSurfaceSignals(channel,true);  // Disconnect previous ones
     StantonSCS3d.softButtonsColor(channel,0x02);  // Make the soft buttons blue
-    switch (currentMode) {    // Special recovery from certain modes
-        case "vinyl":
+    switch (currentMode) {    // Special recovery from certain modesZ
         case "vinyl2":
             // So we don't get stuck at some strange speed when switching from a scratching mode
             engine.scratchDisable(StantonSCS3d.deck);
@@ -1065,10 +1086,14 @@ StantonSCS3d.DeckChangeP2 = function (channel, value) {
 
 StantonSCS3d.S4relative = function (channel, control, value) {
     var currentMode = StantonSCS3d.mode_store["[Channel"+StantonSCS3d.deck+"]"];
-    if (currentMode=="deck") return;   // Skip if in DECK mode
-    if (currentMode=="vinyl" || currentMode=="vinyl2") {
-        var newValue = (value-64);
-        engine.scratchTick(StantonSCS3d.deck,newValue);
+    var newValue=(value-64);
+    switch (currentMode) {
+        case "vinyl":
+            engine.setValue("[Channel"+StantonSCS3d.deck+"]","jog",newValue);
+            break;
+        case "vinyl2":
+            engine.scratchTick(StantonSCS3d.deck,newValue);
+            break;
     }
 }
 
@@ -1142,7 +1167,6 @@ StantonSCS3d.S4absolute = function (channel, control, value) {
             var index = currentMode.charAt(currentMode.length-1);
             if (index != "2" && index != "3") index = "1";
             
-            // We support 36 hot cues but Mixxx only has 31 right now
             if (StantonSCS3d.hotCues[index][button] == -1) return;
             
             if (StantonSCS3d.modifier[currentMode]==1) {
@@ -1240,7 +1264,6 @@ StantonSCS3d.S4touch = function (channel, control, value, status) {
     }
     if ((status & 0xF0) == 0x90) {    // If button down
         switch (currentMode) {
-            case "vinyl":  
             case "vinyl2":
                 engine.scratchEnable(StantonSCS3d.deck, 512, 33+1/3, 1.0/8, (1.0/8)/32);
                     // Recall the cue point if in "scratch & cue" mode only when playing
@@ -1273,9 +1296,9 @@ StantonSCS3d.S4touch = function (channel, control, value, status) {
     }
     // If button up
     switch (currentMode) {
-        case "vinyl":   
         case "vinyl2":
             engine.scratchDisable(StantonSCS3d.deck);
+        case "vinyl":
             var byte1a = 0xB0 + channel;
             midi.sendShortMsg(byte1a,0x01,0x00); //S4 LEDs off
             if (!StantonSCS3d.VUMeters || StantonSCS3d.deck!=1) midi.sendShortMsg(byte1a,0x0C,0x00); //S3 LEDs off
@@ -1470,7 +1493,6 @@ StantonSCS3d.SurfaceButton = function (channel, control, value, status) {
             case "trig3":
                 // Multiple cue points
                 
-                // We support 36 hot cues but Mixxx only has 31 right now                
                 if (StantonSCS3d.hotCues[index][control] == -1) return;
                 
                 if (StantonSCS3d.modifier[currentMode]==1) { // Delete a cue point
