@@ -12,7 +12,7 @@
 #include "cachingreader.h"
 #include "mathstuff.h"
 
-#define NUM_HOT_CUES 32
+#define NUM_HOT_CUES 37
 
 CueControl::CueControl(const char * _group,
                        ConfigObject<ConfigValue> * _config) :
@@ -23,9 +23,11 @@ CueControl::CueControl(const char * _group,
         m_iCurrentlyPreviewingHotcues(0),
         m_iNumHotCues(NUM_HOT_CUES),
         m_pLoadedTrack(),
-        m_mutex(QMutex::Recursive) {
+        m_mutex(QMutex::Recursive),
+        m_bHotcueCancel(false) {
     createControls();
 
+
     m_pTrackSamples = ControlObject::getControl(ConfigKey(_group, "track_samples"));
 
     m_pCuePoint = new ControlObject(ConfigKey(_group, "cue_point"));
@@ -67,6 +69,12 @@ CueControl::CueControl(const char * _group,
     connect(m_pCueDefault, SIGNAL(valueChanged(double)),
             this, SLOT(cueDefault(double)),
             Qt::DirectConnection);
+
+    connect(m_pPlayButton, SIGNAL(valueChanged(double)),
+            this, SLOT(cuePlay(double)),
+            Qt::DirectConnection);
+
+
 }
 
 CueControl::~CueControl() {
@@ -342,6 +350,7 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
 
     if (pCue) {
         if (v) {
+            m_bHotcueCancel = false;
             if (pCue->getPosition() == -1) {
                 hotcueSet(pControl, v);
             } else {
@@ -358,7 +367,15 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
         }
     } else {
         if (v) {
+            // just in case
+            m_bHotcueCancel = false;
             hotcueSet(pControl, v);
+        } else if (m_bPreviewingHotcue) {
+            // The cue is non-existent, yet we got a release for it and are
+            // currently previewing a hotcue. This is indicative of a corner
+            // case where the cue was detached while we were pressing it. Let
+            // hotcueActivatePreview handle it.
+            hotcueActivatePreview(pControl, v);
         }
     }
 }
@@ -375,23 +392,41 @@ void CueControl::hotcueActivatePreview(HotcueControl* pControl, double v) {
             int iPosition = pCue->getPosition();
             m_pPlayButton->set(1.0);
             m_bPreviewingHotcue = true;
+            pControl->setPreviewing(true);
+            pControl->setPreviewingPosition(iPosition);
 
             // Need to unlock before emitting any signals to prevent deadlock.
             lock.unlock();
 
             emit(seekAbs(iPosition));
         }
-    } else {
-        if (m_bPreviewingHotcue && pCue && pCue->getPosition() != -1) {
+    } else if (m_bPreviewingHotcue) {
+        // This is a activate release and we are previewing at least one
+        // hotcue. If this hotcue is previewing:
+        if (pControl->isPreviewing()) {
+            // Mark this hotcue as not previewing.
+            int iPosition = pControl->getPreviewingPosition();
+            pControl->setPreviewing(false);
+            pControl->setPreviewingPosition(-1);
+
+            // If this is the last hotcue to leave preview.
             if (--m_iCurrentlyPreviewingHotcues == 0) {
-                int iPosition = pCue->getPosition();
-                m_pPlayButton->set(0.0);
+                bool bHotcueCancel = m_bHotcueCancel;
                 m_bPreviewingHotcue = false;
-
-                // Need to unlock before emitting any signals to prevent deadlock.
-                lock.unlock();
-
-                emit(seekAbs(iPosition));
+                m_bHotcueCancel = false;
+                // If hotcue cancel is marked then do not snap back to the
+                // hotcue and stop. Otherwise, seek back to the start point and
+                // stop.
+                if (bHotcueCancel) {
+                    // Re-trigger the play button value so controllers get the correct one
+                    // after.
+                    m_pPlayButton->set(m_pPlayButton->get());
+                } else {
+                    m_pPlayButton->set(0.0);
+                    // Need to unlock before emitting any signals to prevent deadlock.
+                    lock.unlock();
+                    emit(seekAbs(iPosition));
+                }
             }
         }
     }
@@ -425,7 +460,12 @@ void CueControl::hotcuePositionChanged(HotcueControl* pControl, double newPositi
             pCue->setHotCue(-1);
             detachCue(pControl->getHotcueNumber());
         } else if (newPosition > 0 && newPosition < m_pTrackSamples->get()) {
-            pCue->setPosition(newPosition);
+            int position = newPosition;
+            // People writing from MIDI land, elsewhere might be careless.
+            if (position % 2 != 0) {
+                position--;
+            }
+            pCue->setPosition(position);
         }
     }
 }
@@ -558,7 +598,7 @@ void CueControl::cueCDJ(double v) {
      * If pressed while playing, stop playback at go to cue.
      * If pressed while stopped and at cue, play while pressed.
      * If pressed while stopped and not at cue, set new cue point.
-     * TODO: If play is pressed while holding cue, the deck is now playing.
+     * If play is pressed while holding cue, the deck is now playing. (Handled in cuePlay().)
      */
 
     QMutexLocker lock(&m_mutex);
@@ -595,6 +635,28 @@ void CueControl::cueCDJ(double v) {
 
         emit(seekAbs(cuePoint));
     }
+    else {
+        // Re-trigger the play button value so controllers get the correct one
+        // after cuePlay() changes it.
+        m_pPlayButton->set(m_pPlayButton->get());
+    }
+}
+
+void CueControl::cuePlay(double v) {
+    QMutexLocker lock(&m_mutex);
+
+    if (m_bPreviewing && !v) {
+    // we're previewing? Then stop previewing and go into normal play mode.
+        m_pPlayButton->set(1.0);
+        m_bPreviewing = false;
+    }
+
+    if (m_bPreviewingHotcue && !v) {
+        m_pPlayButton->set(1.0);
+        m_bHotcueCancel = true;
+    }
+
+    lock.unlock();
 }
 
 void CueControl::cueDefault(double v) {
@@ -617,7 +679,9 @@ ConfigKey HotcueControl::keyForControl(int hotcue, QString name) {
 HotcueControl::HotcueControl(const char* pGroup, int i)
         : m_pGroup(pGroup),
           m_iHotcueNumber(i),
-          m_pCue(NULL) {
+          m_pCue(NULL),
+          m_bPreviewing(false),
+          m_iPreviewingPosition(-1) {
     m_hotcuePosition = new ControlObject(keyForControl(i, "position"));
     connect(m_hotcuePosition, SIGNAL(valueChanged(double)),
             this, SLOT(slotHotcuePositionChanged(double)),
