@@ -18,24 +18,24 @@
 #include <QEvent>
 #include <QtDebug>
 
-#include "enginebuffer.h"
+#include "engine/enginebuffer.h"
 #include "cachingreader.h"
 
 #include "controlpushbutton.h"
 #include "controlobjectthreadmain.h"
 #include "configobject.h"
 #include "controlpotmeter.h"
-#include "enginebufferscalest.h"
-#include "enginebufferscalelinear.h"
-#include "enginebufferscalereal.h"
-#include "enginebufferscaledummy.h"
+#include "engine/enginebufferscalest.h"
+#include "engine/enginebufferscalelinear.h"
+#include "engine/enginebufferscalereal.h"
+#include "engine/enginebufferscaledummy.h"
 #include "mathstuff.h"
 #include "engine/engineworkerscheduler.h"
 #include "engine/readaheadmanager.h"
 #include "engine/enginecontrol.h"
-#include "loopingcontrol.h"
-#include "ratecontrol.h"
-#include "bpmcontrol.h"
+#include "engine/loopingcontrol.h"
+#include "engine/ratecontrol.h"
+#include "engine/bpmcontrol.h"
 
 #include "trackinfoobject.h"
 
@@ -83,7 +83,7 @@ EngineBuffer::EngineBuffer(const char * _group, ConfigObject<ConfigValue> * _con
     playButtonCOT = new ControlObjectThreadMain(playButton);
 
     //Play from Start Button (for sampler)
-    playStartButton = new ControlPushButton(ConfigKey(group, "playstart"));
+    playStartButton = new ControlPushButton(ConfigKey(group, "start_play"));
     connect(playStartButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlPlayFromStart(double)),
             Qt::DirectConnection);
@@ -320,6 +320,11 @@ void EngineBuffer::slotTrackLoadFailed(TrackPointer pTrack,
 }
 
 void EngineBuffer::ejectTrack() {
+    // Don't allow ejections while playing a track. We don't need to lock to
+    // call ControlObject::get() so this is fine.
+    if (playButton->get() > 0)
+        return;
+
     pause.lock();
     TrackPointer pTrack = m_pCurrentTrack;
     m_pCurrentTrack.clear();
@@ -373,29 +378,41 @@ void EngineBuffer::slotControlSeekAbs(double abs)
     slotControlSeek(abs/file_length_old);
 }
 
-void EngineBuffer::slotControlPlay(double)
+void EngineBuffer::slotControlPlay(double v)
 {
+    // If no track is currently loaded, turn play off.
+    if (v > 0.0 && !m_pCurrentTrack) {
+        playButton->set(0.0f);
+    }
 }
 
-void EngineBuffer::slotControlStart(double)
+void EngineBuffer::slotControlStart(double v)
 {
-    slotControlSeek(0.);
+    if (v > 0.0) {
+        slotControlSeek(0.);
+    }
 }
 
-void EngineBuffer::slotControlEnd(double)
+void EngineBuffer::slotControlEnd(double v)
 {
-    slotControlSeek(1.);
+    if (v > 0.0) {
+        slotControlSeek(1.);
+    }
 }
 
-void EngineBuffer::slotControlPlayFromStart(double)
+void EngineBuffer::slotControlPlayFromStart(double v)
 {
-    slotControlSeek(0.);
-    playButton->set(1);
+    if (v > 0.0) {
+        slotControlSeek(0.);
+        playButton->set(1);
+    }
 }
 
-void EngineBuffer::slotControlStop(double)
+void EngineBuffer::slotControlStop(double v)
 {
-    playButton->set(0);
+    if (v > 0.0) {
+        playButton->set(0);
+    }
 }
 
 void EngineBuffer::process(const CSAMPLE *, const CSAMPLE * pOut, const int iBufferSize)
