@@ -1,24 +1,30 @@
-
 #include <QtDebug>
 #include <QtCore>
 #include <QtSql>
+
 #include "trackinfoobject.h"
 #include "library/dao/trackdao.h"
 #include "audiotagger.h"
+#include "soundsourceproxy.h"
 
 // The number of tracks to cache in memory at once. Once the n+1'th track is
 // created, the TrackDAO's QCache deletes its TrackPointer to the track, which
 // allows the track reference count to drop to zero. The track cache basically
 // functions to hold a reference to the track so its reference count stays above
 // 0.
-#define TRACK_CACHE_SIZE 20
+#define TRACK_CACHE_SIZE 5
 
-TrackDAO::TrackDAO(QSqlDatabase& database, CueDAO& cueDao, ConfigObject<ConfigValue> * pConfig)
+TrackDAO::TrackDAO(QSqlDatabase& database,
+                   CueDAO& cueDao,
+                   PlaylistDAO& playlistDao,
+                   CrateDAO& crateDao,
+                   ConfigObject<ConfigValue> * pConfig)
         : m_database(database),
           m_cueDao(cueDao),
+          m_playlistDao(playlistDao),
+          m_crateDao(crateDao),
           m_pConfig(pConfig),
           m_trackCache(TRACK_CACHE_SIZE) {
-
 }
 
 void TrackDAO::finish()
@@ -35,6 +41,7 @@ void TrackDAO::finish()
 
 TrackDAO::~TrackDAO()
 {
+    qDebug() << "~TrackDAO()";
 }
 
 void TrackDAO::initialize()
@@ -109,25 +116,29 @@ void TrackDAO::saveTrack(TrackInfoObject* pTrack) {
     if (!pTrack) {
         qWarning() << "TrackDAO::saveTrack() was given NULL track.";
     }
+    //qDebug() << "TrackDAO::saveTrack" << pTrack->getId() << pTrack->getInfo();
     // If track's id is not -1, then update, otherwise add.
     int trackId = pTrack->getId();
     if (trackId != -1) {
         if (pTrack->isDirty()) {
-            updateTrack(pTrack);
-            m_dirtyTracks.remove(trackId);
+            if (!m_dirtyTracks.contains(trackId)) {
+                qDebug() << "WARNING: Inconsistent state in TrackDAO. Track is dirty while TrackDAO thinks it is clean.";
+            }
 
-            //Write audio meta data, if enabled in the preferences
+            //qDebug() << this << "Dirty tracks before claen save:" << m_dirtyTracks.size();
+            //qDebug() << "TrackDAO::saveTrack. Dirty. Calling update";
+            updateTrack(pTrack);
+
+            // Write audio meta data, if enabled in the preferences
             writeAudioMetaData(pTrack);
 
-            emit(trackClean(trackId));
+            //qDebug() << this << "Dirty tracks remaining after clean save:" << m_dirtyTracks.size();
         } else {
-            // We don't know why this is happening right now, but this assert is
-            //failing. The condition is mostly harmless, so simply take
-            //corrective action instead of asserting and warn about it in the
-            //log -- rryan 12/2010
+            //qDebug() << "TrackDAO::saveTrack. Not Dirty";
+            //qDebug() << this << "Dirty tracks remaining:" << m_dirtyTracks.size();
 
             // Q_ASSERT(!m_dirtyTracks.contains(trackId));
-            if (!m_dirtyTracks.contains(trackId)) {
+            if (m_dirtyTracks.contains(trackId)) {
                 qDebug() << "WARNING: Inconsistent state in TrackDAO. Track is clean while TrackDAO thinks it is dirty. Correcting.";
                 m_dirtyTracks.remove(trackId);
             }
@@ -135,7 +146,7 @@ void TrackDAO::saveTrack(TrackInfoObject* pTrack) {
             //qDebug() << "Skipping track update for track" << pTrack->getId();
         }
     } else {
-        addTrack(pTrack);
+        addTrack(pTrack, false);
     }
 }
 
@@ -143,13 +154,13 @@ bool TrackDAO::isDirty(int trackId) {
     return m_dirtyTracks.contains(trackId);
 }
 
-void TrackDAO::slotTrackDirty() {
+void TrackDAO::slotTrackDirty(TrackInfoObject* pTrack) {
+    //qDebug() << "TrackDAO::slotTrackDirty" << pTrack->getInfo();
     // This is a private slot that is connected to TIO's created by this
     // TrackDAO. It is a way for the track to ask that it be saved. The only
     // time this could be unsafe is when the TIO's reference count drops to
     // 0. When that happens, the TIO is deleted with QObject:deleteLater, so Qt
     // will wait for this slot to comlete.
-    TrackInfoObject* pTrack = dynamic_cast<TrackInfoObject*>(sender());
     if (pTrack) {
         int id = pTrack->getId();
         if (id != -1) {
@@ -159,13 +170,30 @@ void TrackDAO::slotTrackDirty() {
     }
 }
 
-void TrackDAO::slotTrackChanged() {
+void TrackDAO::slotTrackClean(TrackInfoObject* pTrack) {
+    //qDebug() << "TrackDAO::slotTrackClean" << pTrack->getInfo();
+    // This is a private slot that is connected to TIO's created by this
+    // TrackDAO. It is a way for the track to ask that it be saved. The only
+    // time this could be unsafe is when the TIO's reference count drops to
+    // 0. When that happens, the TIO is deleted with QObject:deleteLater, so Qt
+    // will wait for this slot to comlete.
+
+    if (pTrack) {
+        int id = pTrack->getId();
+        if (id != -1) {
+            m_dirtyTracks.remove(id);
+            emit(trackClean(id));
+        }
+    }
+}
+
+void TrackDAO::slotTrackChanged(TrackInfoObject* pTrack) {
+    //qDebug() << "TrackDAO::slotTrackChanged" << pTrack->getInfo();
     // This is a private slot that is connected to TIO's created by this
     // TrackDAO. It is a way for the track to ask that it be saved. The only
     // time this could be unsafe is when the TIO's reference count drops to
     // 0. When that happens, the TIO is deleted with QObject:deleteLater, so Qt
     // will wait for this slot to comlete.
-    TrackInfoObject* pTrack = dynamic_cast<TrackInfoObject*>(sender());
     if (pTrack) {
         int id = pTrack->getId();
         if (id != -1) {
@@ -174,19 +202,20 @@ void TrackDAO::slotTrackChanged() {
     }
 }
 
-void TrackDAO::slotTrackSave() {
+void TrackDAO::slotTrackSave(TrackInfoObject* pTrack) {
+    //qDebug() << "TrackDAO::slotTrackSave" << pTrack->getId() << pTrack->getInfo();
     // This is a private slot that is connected to TIO's created by this
     // TrackDAO. It is a way for the track to ask that it be saved. The last
     // time it is used is when the track is being deleted (i.e. its reference
     // count has dropped to 0). The TIO is deleted with QObject:deleteLater, so
     // Qt will wait for this slot to comlete.
-    TrackInfoObject* pTrack = dynamic_cast<TrackInfoObject*>(sender());
     if (pTrack) {
         saveTrack(pTrack);
     }
 }
 
 void TrackDAO::saveDirtyTracks() {
+    qDebug() << "TrackDAO::saveDirtyTracks()";
     QHashIterator<int, TrackWeakPointer> it(m_tracks);
     while (it.hasNext()) {
         it.next();
@@ -196,6 +225,7 @@ void TrackDAO::saveDirtyTracks() {
             saveTrack(pTrack);
         }
     }
+    clearCache();
 }
 
 void TrackDAO::prepareTrackLocationsInsert(QSqlQuery& query) {
@@ -256,11 +286,12 @@ void TrackDAO::bindTrackToLibraryInsert(QSqlQuery& query, TrackInfoObject* pTrac
     query.bindValue(":header_parsed", pTrack->getHeaderParsed() ? 1 : 0);
 }
 
-void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd) {
+void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd, bool unremove) {
+    QSet<int> tracksAddedSet;
     QTime time;
     time.start();
 
-    //Start the transaction
+    // Start the transaction
     m_database.transaction();
 
     QSqlQuery query(m_database);
@@ -270,22 +301,27 @@ void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd) {
     // Major time saver for having this outside the loop
     prepareTrackLocationsInsert(query);
 
+    QStringList trackLocationIds;
     foreach (TrackInfoObject* pTrack, tracksToAdd) {
+        if (pTrack == NULL || !isTrackFormatSupported(pTrack)) {
+            // TODO(XXX) provide some kind of error code on a per-track basis.
+            continue;
+        }
         bindTrackToTrackLocationsInsert(query, pTrack);
 
         int trackLocationId = -1;
         if (!query.exec()) {
             qDebug() << "Location " << pTrack->getLocation() << " is already in the DB";
-            query.bindValue(":location", pTrack->getLocation());
+            query_finder.bindValue(":location", pTrack->getLocation());
 
-            if (!query.exec()) {
-                // We can't even select this, something is wrong.
-                qDebug() << query.lastError();
-                m_database.rollback();
-                return;
+            if (!query_finder.exec()) {
+                // We can't even select this, something is wrong. Skip this
+                // track -- maybe we'll have luck with others.
+                qDebug() << query_finder.lastError();
+                continue;
             }
-            while (query.next()) {
-                trackLocationId = query.value(query.record().indexOf("id")).toInt();
+            while (query_finder.next()) {
+                trackLocationId = query_finder.value(query_finder.record().indexOf("id")).toInt();
             }
         } else {
             // Inserting succeeded, so just get the last rowid.
@@ -297,140 +333,122 @@ void TrackDAO::addTracks(QList<TrackInfoObject*> tracksToAdd) {
         // track. This takes advantage of the fact that I know the
         // LibraryScanner doesn't use these tracks for anything. rryan 9/2010
         pTrack->setId(trackLocationId);
+        trackLocationIds.append(QString::number(trackLocationId));
+    }
+
+    // Look up pre-existing library records for the track location ids.
+    QSqlQuery track_lookup(m_database);
+    // Mapping of track library record id to mixxx_deleted field.
+    QHash<int, QPair<int, bool> > tracksPresent;
+    track_lookup.prepare("SELECT location, id, mixxx_deleted from library WHERE location IN (" +
+                         trackLocationIds.join(",")+ ")");
+    if (!track_lookup.exec()) {
+        qDebug() << __FILE__ << __LINE__ << "Failed to lookup existing tracks:"
+                 << track_lookup.lastError()
+                 << track_lookup.executedQuery();
+    } else {
+        QSqlRecord track_lookup_record = track_lookup.record();
+        int locationIdColumn = track_lookup_record.indexOf("location");
+        int idColumn = track_lookup_record.indexOf("id");
+        int mixxxDeletedColumn = track_lookup_record.indexOf("mixxx_deleted");
+        while (track_lookup.next()) {
+            int locationId = track_lookup.value(locationIdColumn).toInt();
+            int trackId = track_lookup.value(idColumn).toInt();
+            bool removed = track_lookup.value(mixxxDeletedColumn).toBool();
+            tracksPresent[locationId] = QPair<int, bool>(trackId, removed);
+        }
     }
 
     // Major time saver for having this outside the loop
     prepareLibraryInsert(query);
 
     foreach (TrackInfoObject* pTrack, tracksToAdd) {
-        bindTrackToLibraryInsert(query, pTrack, pTrack->getId());
+        // Skip tracks that did not make it past the previous part.
+        if (pTrack == NULL || pTrack->getId() < 0) {
+            continue;
+        }
+
+        // Immediately undo the hack we did above so we do not accidentally
+        // leave the ID incorrectly set.
+        int locationId = pTrack->getId();
+        pTrack->setId(-1);
+
+        // Skip tracks that are already in the database. Optionally unremove
+        // them.
+        QHash<int, QPair<int, bool> >::const_iterator it = tracksPresent.find(locationId);
+        if (it != tracksPresent.end()) {
+            int trackId = it.value().first;
+            bool removed = it.value().second;
+            if (removed && unremove) {
+                QSqlQuery unremove_query(m_database);
+                unremove_query.prepare("UPDATE library SET mixxx_deleted = 0 WHERE id = :id");
+                unremove_query.bindValue(":id", trackId);
+                if (!unremove_query.exec()) {
+                    qDebug() << __FILE__ << __LINE__ << "Could not unremove track" << trackId
+                             << unremove_query.lastError()
+                             << unremove_query.executedQuery();
+                } else {
+                    tracksAddedSet.insert(trackId);
+                }
+            }
+
+            // Regardless of whether we unremoved this track or not -- it's
+            // already in the library and so we need to skip it. Set the track's
+            // trackId so the caller can know it. TODO(XXX) this is a little
+            // weird because the track has whatever metadata the caller supplied
+            // and that metadata may differ from what is already in the
+            // database. I'm ignoring this corner case. rryan 10/2011
+            pTrack->setId(trackId);
+            continue;
+        }
+
+        bindTrackToLibraryInsert(query, pTrack, locationId);
 
         if (!query.exec()) {
+            // We failed to insert the track. Maybe it is already in the library
+            // but marked deleted? Skip this track.
             qDebug() << "Failed to INSERT new track into library"
                      << __FILE__ << __LINE__ << query.lastError();
-            m_database.rollback();
-            return;
+            continue;
         }
-
         int trackId = query.lastInsertId().toInt();
-
-        if (trackId >= 0) {
-            m_cueDao.saveTrackCues(trackId, pTrack);
-        } else {
-            qDebug() << "Could not get track ID to save the track cue points:"
-                     << query.lastError();
-        }
-
-        // Undo the hack we did above by setting this track to its new id in the
-        // database.
         pTrack->setId(trackId);
+        m_cueDao.saveTrackCues(trackId, pTrack);
+        pTrack->setDirty(false);
+        tracksAddedSet.insert(trackId);
     }
 
     m_database.commit();
 
-    qDebug() << "Writing tracks to database by TrackDAO::addTracks() took " << time.elapsed() << " ms";
+    qDebug() << this << "addTracks took" << time.elapsed() << "ms to add"
+             << tracksAddedSet.size() << "tracks";
+    if (tracksAddedSet.size() > 0) {
+        emit(tracksAdded(tracksAddedSet));
+    }
 }
 
-int TrackDAO::addTrack(QFileInfo& fileInfo) {
+int TrackDAO::addTrack(QFileInfo& fileInfo, bool unremove) {
     int trackId = -1;
     TrackInfoObject * pTrack = new TrackInfoObject(fileInfo);
     if (pTrack) {
         //Add the song to the database.
-        addTrack(pTrack);
+        addTrack(pTrack, unremove);
         trackId = pTrack->getId();
         delete pTrack;
     }
     return trackId;
 }
 
-int TrackDAO::addTrack(QString absoluteFilePath)
+int TrackDAO::addTrack(QString absoluteFilePath, bool unremove)
 {
     QFileInfo fileInfo(absoluteFilePath);
-    return addTrack(fileInfo);
+    return addTrack(fileInfo, unremove);
 }
 
-void TrackDAO::addTrack(TrackInfoObject* pTrack)
-{
-    QTime time;
-    time.start();
-
-    //qDebug() << "TrackDAO::addTrack" << QThread::currentThread() << m_database.connectionName();
-    //qDebug() << "TrackCollection::addTrack(), inserting into DB";
-    Q_ASSERT(pTrack); //Why you be giving me NULL pTracks
-
-    //Start the transaction
-    m_database.transaction();
-
-    QSqlQuery query(m_database);
-    int trackLocationId = -1;
-
-    // Insert the track location into the corresponding table. This will fail
-    // silently if the location is already in the table because it has a UNIQUE
-    // constraint.
-    prepareTrackLocationsInsert(query);
-    bindTrackToTrackLocationsInsert(query, pTrack);
-
-    if (!query.exec()) {
-        // Inserting into track_locations failed, so the file already
-        // exists. Query for its id.
-        query.prepare("SELECT id FROM track_locations WHERE location=:location");
-        query.bindValue(":location", pTrack->getLocation());
-
-        if (!query.exec()) {
-            // We can't even select this, something is wrong.
-            qDebug() << query.lastError();
-            m_database.rollback();
-            return;
-        }
-        while (query.next()) {
-            trackLocationId = query.value(query.record().indexOf("id")).toInt();
-        }
-    } else {
-        // Inserting succeeded, so just get the last rowid.
-        QVariant lastInsert = query.lastInsertId();
-        trackLocationId = lastInsert.toInt();
-    }
-
-    //Failure of this assert indicates that we were unable to insert the track
-    //location into the table AND we could not retrieve the id of that track
-    //location from the same table. "It shouldn't happen"... unless I screwed up
-    //- Albert :)
-    Q_ASSERT(trackLocationId >= 0);
-
-    prepareLibraryInsert(query);
-    bindTrackToLibraryInsert(query, pTrack, trackLocationId);
-
-    int trackId = -1;
-
-    if (!query.exec())
-    {
-        qDebug() << "Failed to INSERT new track into library"
-                 << __FILE__ << __LINE__ << query.lastError();
-        m_database.rollback();
-        return;
-    } else {
-        // Inserting succeeded, so just get the last rowid.
-        trackId = query.lastInsertId().toInt();
-    }
-    //query.finish();
-
-    Q_ASSERT(trackId >= 0);
-
-    if (trackId >= 0) {
-        m_cueDao.saveTrackCues(trackId, pTrack);
-    } else {
-        qDebug() << "Could not get track ID to save the track cue points:"
-                 << query.lastError();
-    }
-
-    pTrack->setId(trackId);
-    pTrack->setDirty(false);
-
-    //query.finish();
-
-    //Commit the transaction
-    m_database.commit();
-    //qDebug() << "addTrack took" << time.elapsed() << "ms";
+void TrackDAO::addTrack(TrackInfoObject* pTrack, bool unremove) {
+    QList<TrackInfoObject*> tracksToAdd;
+    tracksToAdd.push_back(pTrack);
+    addTracks(tracksToAdd, unremove);
 }
 
   /** Removes a track from the library track collection. */
@@ -440,10 +458,14 @@ void TrackDAO::removeTrack(int id)
     Q_ASSERT(id >= 0);
     QSqlQuery query(m_database);
 
+    // Remove track from crates and playlists.
+    m_playlistDao.removeTrackFromPlaylists(id);
+    m_crateDao.removeTrackFromCrates(id);
+
     //Mark the track as deleted!
     query.prepare("UPDATE library "
                   "SET mixxx_deleted=1 "
-                  "WHERE id = " + QString("%1").arg(id));
+                  "WHERE id = " + QString::number(id));
     query.exec();
     //query.finish();
 
@@ -451,6 +473,10 @@ void TrackDAO::removeTrack(int id)
     if (query.lastError().isValid()) {
         qDebug() << query.lastError();
     }
+
+    QSet<int> tracksRemovedSet;
+    tracksRemovedSet.insert(id);
+    emit(tracksRemoved(tracksRemovedSet));
 }
 
 void TrackDAO::removeTracks(QList<int> ids) {
@@ -470,6 +496,9 @@ void TrackDAO::removeTracks(QList<int> ids) {
     if (!query.exec()) {
         qDebug() << query.lastError();
     }
+
+    QSet<int> tracksRemovedSet = QSet<int>::fromList(ids);
+    emit(tracksRemoved(tracksRemovedSet));
 }
 
 /*** If a track has been manually "removed" from Mixxx's library by the user via
@@ -489,27 +518,25 @@ void TrackDAO::unremoveTrack(int trackId)
     if (!query.exec()) {
         qDebug() << "Failed to set track" << trackId << "as undeleted" << query.lastError();
     }
+    QSet<int> tracksAddedSet;
+    tracksAddedSet.insert(trackId);
+    emit(tracksAdded(tracksAddedSet));
 }
 
 // static
 void TrackDAO::deleteTrack(TrackInfoObject* pTrack) {
     Q_ASSERT(pTrack);
 
-    //qDebug() << "Got deletion call for track" << pTrack << "ID" << pTrack->getId() << pTrack->getInfo();
+    //qDebug() << "Garbage Collecting" << pTrack << "ID" << pTrack->getId() << pTrack->getInfo();
 
-    // Save dirty tracks.
-    pTrack->save();
-
-    // if (iId != -1 && isDirty(iId)) {
-    //     saveTrack(track);
-    // }
+    // Save the track if it is dirty.
+    pTrack->doSave();
 
     // Now Qt will delete it in the event loop.
     pTrack->deleteLater();
 }
 
-TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const
-{
+TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const {
     if (!query.isValid()) {
         //query.exec();
     }
@@ -548,6 +575,7 @@ TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const
         QString filetype = query.value(query.record().indexOf("filetype")).toString();
         QString location = query.value(query.record().indexOf("location")).toString();
         bool header_parsed = query.value(query.record().indexOf("header_parsed")).toBool();
+        QDateTime date_created = query.value(query.record().indexOf("datetime_added")).toDateTime();
 
         TrackInfoObject* track = new TrackInfoObject(location, false);
 
@@ -581,19 +609,26 @@ TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const
         track->setType(filetype);
         track->setLocation(location);
         track->setHeaderParsed(header_parsed);
+        track->setDateAdded(date_created);
 
         track->setCuePoints(m_cueDao.getCuesForTrack(trackId));
         track->setDirty(false);
 
         // Listen to dirty and changed signals
-        connect(track, SIGNAL(dirty()),
-                this, SLOT(slotTrackDirty()));
-        connect(track, SIGNAL(changed()),
-                this, SLOT(slotTrackChanged()));
-        connect(track, SIGNAL(save()),
-                this, SLOT(slotTrackSave()));
+        connect(track, SIGNAL(dirty(TrackInfoObject*)),
+                this, SLOT(slotTrackDirty(TrackInfoObject*)),
+                Qt::DirectConnection);
+        connect(track, SIGNAL(clean(TrackInfoObject*)),
+                this, SLOT(slotTrackClean(TrackInfoObject*)),
+                Qt::DirectConnection);
+        connect(track, SIGNAL(changed(TrackInfoObject*)),
+                this, SLOT(slotTrackChanged(TrackInfoObject*)),
+                Qt::DirectConnection);
+        connect(track, SIGNAL(save(TrackInfoObject*)),
+                this, SLOT(slotTrackSave(TrackInfoObject*)),
+                Qt::DirectConnection);
 
-        TrackPointer pTrack = TrackPointer(track, this->deleteTrack);
+        TrackPointer pTrack = TrackPointer(track, &TrackDAO::deleteTrack);
 
         // Automatic conversion to a weak pointer
         m_tracks[trackId] = pTrack;
@@ -613,7 +648,7 @@ TrackPointer TrackDAO::getTrackFromDB(QSqlQuery &query) const
     return TrackPointer();
 }
 
-TrackPointer TrackDAO::getTrack(int id) const
+TrackPointer TrackDAO::getTrack(int id, bool cacheOnly) const
 {
     //qDebug() << "TrackDAO::getTrack" << QThread::currentThread() << m_database.connectionName();
 
@@ -621,7 +656,7 @@ TrackPointer TrackDAO::getTrack(int id) const
     // to the track. We do this first so that the QCache keeps track of the
     // least-recently-used track so that it expires them intelligently.
     if (m_trackCache.contains(id)) {
-        TrackPointer& pTrack = *m_trackCache[id];
+        TrackPointer pTrack = *m_trackCache[id];
 
         // If the strong reference is still valid (it should be), then return
         // it. Otherwise query the DB for the track.
@@ -643,11 +678,17 @@ TrackPointer TrackDAO::getTrack(int id) const
             return pTrack;
     }
 
+    // The person only wanted the track if it was cached.
+    if (cacheOnly) {
+        //qDebug() << "TrackDAO::getTrack()" << id << "Caller wanted track but only if it was cached. Returning null.";
+        return TrackPointer();
+    }
+
     QTime time;
     time.start();
     QSqlQuery query(m_database);
 
-    query.prepare("SELECT library.id, artist, title, album, year, genre, tracknumber, filetype, rating, key, track_locations.location as location, track_locations.filesize as filesize, comment, url, duration, bitrate, samplerate, cuepoint, bpm, replaygain, wavesummaryhex, channels, header_parsed, timesplayed, played FROM Library INNER JOIN track_locations ON library.location = track_locations.id WHERE library.id=" + QString("%1").arg(id));
+    query.prepare("SELECT library.id, artist, title, album, year, genre, tracknumber, filetype, rating, key, track_locations.location as location, track_locations.filesize as filesize, comment, url, duration, bitrate, samplerate, cuepoint, bpm, replaygain, wavesummaryhex, channels, header_parsed, timesplayed, played, datetime_added FROM Library INNER JOIN track_locations ON library.location = track_locations.id WHERE library.id=" + QString("%1").arg(id));
 
     TrackPointer pTrack;
 
@@ -909,3 +950,7 @@ void TrackDAO::writeAudioMetaData(TrackInfoObject* pTrack){
         tagger.save();
     }
 }
+
+bool TrackDAO::isTrackFormatSupported(TrackInfoObject* pTrack) const {
+    return SoundSourceProxy::isFilenameSupported(pTrack->getFilename());
+}
