@@ -29,8 +29,9 @@ LibraryScanner::LibraryScanner(TrackCollection* collection) :
     m_pCollection(collection),
     m_libraryHashDao(m_database),
     m_cueDao(m_database),
-    m_trackDao(m_database, m_cueDao),
     m_playlistDao(m_database),
+    m_crateDao(m_database),
+    m_trackDao(m_database, m_cueDao, m_playlistDao, m_crateDao),
     //Don't initialize m_database here, we need to do it in run() so the DB conn is in
     //the right thread.
     nameFilters(SoundSourceProxy::supportedFileExtensionsString().split(" "))
@@ -232,8 +233,8 @@ void LibraryScanner::run()
      * of 3-4 !!!
      */
 
-    // Runs inside a transaction
-    m_trackDao.addTracks(tracksToAdd);
+    // Runs inside a transaction. Do not unremove files.
+    m_trackDao.addTracks(tracksToAdd, false);
 
     QMutableListIterator<TrackInfoObject*> it(tracksToAdd);
     while (it.hasNext()) {
