@@ -20,10 +20,6 @@
 #include <QtGui>
 #include <QTranslator>
 
-#include "widget/wknob.h"
-#include "widget/wslider.h"
-#include "widget/wpushbutton.h"
-#include "widget/woverview.h"
 #include "mixxx.h"
 #include "controlnull.h"
 #include "controlpotmeter.h"
@@ -264,6 +260,15 @@ MixxxApp::MixxxApp(QApplication *a, struct CmdlineArgs args)
             hasChanged_MusicDir = true;
         }
     }
+    /*
+     * Do not write meta data back to ID3 when meta data has changed
+     * Because multiple TrackDao objects can exists for a particular track
+     * writing meta data may ruine your MP3 file if done simultaneously.
+     * see Bug #728197
+     * For safety reasons, we deactivate this feature.
+     */
+    m_pConfig->set(ConfigKey("[Library]","WriteAudioTags"), ConfigValue(0));
+
 
     // library dies in seemingly unrelated qtsql error about not having a
     // sqlite driver if this path doesn't exist. Normally config->Save()
@@ -294,11 +299,20 @@ MixxxApp::MixxxApp(QApplication *a, struct CmdlineArgs args)
 
     //Scan the library for new files and directories
     bool rescan = (bool)m_pConfig->getValueString(ConfigKey("[Library]","RescanOnStartup")).toInt();
+    // rescan the library if we get a new plugin
+    QSet<QString> prev_plugins = QSet<QString>::fromList(m_pConfig->getValueString(
+        ConfigKey("[Library]", "SupportedFileExtensions")).split(",", QString::SkipEmptyParts));
+    QSet<QString> curr_plugins = QSet<QString>::fromList(
+        SoundSourceProxy::supportedFileExtensions());
+    rescan = rescan || (prev_plugins != curr_plugins);
+
     if(rescan || hasChanged_MusicDir){
         m_pLibraryScanner->scan(
             m_pConfig->getValueString(ConfigKey("[Playlist]", "Directory")));
         qDebug() << "Rescan finished";
     }
+    m_pConfig->set(ConfigKey("[Library]", "SupportedFileExtensions"),
+        QStringList(SoundSourceProxy::supportedFileExtensions()).join(","));
 
     // Call inits to invoke all other construction parts
 
@@ -395,7 +409,7 @@ MixxxApp::MixxxApp(QApplication *a, struct CmdlineArgs args)
             m_pLibrary->getTracksToAutoLoad();
         for (int i = 0; i < (int)m_pPlayerManager->numDecks()
                 && i < tracksToAutoLoad.count(); i++) {
-            m_pPlayerManager->slotLoadToDeck(args.qlMusicFiles.at(i), i+1);
+            m_pPlayerManager->slotLoadToDeck(tracksToAutoLoad.at(i)->getLocation(), i+1);
         }
     }
 
@@ -694,8 +708,9 @@ void MixxxApp::initActions()
     m_pOptionsPreferences->setShortcut(tr("Ctrl+P"));
     m_pOptionsPreferences->setShortcutContext(Qt::ApplicationShortcut);
 
-    m_pHelpAboutApp = new QAction(tr("&About..."), this);
-    m_pHelpSupport = new QAction(tr("&Community Support..."), this);
+    m_pHelpAboutApp = new QAction(tr("&About"), this);
+    m_pHelpSupport = new QAction(tr("&Community Support"), this);
+    m_pHelpFeedback = new QAction(tr("Send Us &Feedback"), this);
 
 #ifdef __VINYLCONTROL__
     m_pOptionsVinylControl = new QAction(tr("Enable &Vinyl Control"), this);
@@ -822,6 +837,10 @@ void MixxxApp::initActions()
     m_pHelpSupport->setWhatsThis(tr("Support\n\nGet help with Mixxx"));
     connect(m_pHelpSupport, SIGNAL(triggered()), this, SLOT(slotHelpSupport()));
 
+    m_pHelpFeedback->setStatusTip(tr("Send feedback to the Mixxx team."));
+    m_pHelpFeedback->setWhatsThis(tr("Support\n\nSend feedback to the Mixxx team."));
+    connect(m_pHelpFeedback, SIGNAL(triggered()), this, SLOT(slotHelpFeedback()));
+
     m_pHelpAboutApp->setStatusTip(tr("About the application"));
     m_pHelpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
     connect(m_pHelpAboutApp, SIGNAL(triggered()), this, SLOT(slotHelpAbout()));
@@ -880,6 +899,7 @@ void MixxxApp::initMenuBar()
 
     // menuBar entry helpMenu
     m_pHelpMenu->addAction(m_pHelpSupport);
+    m_pHelpMenu->addAction(m_pHelpFeedback);
     m_pHelpMenu->addSeparator();
     m_pHelpMenu->addAction(m_pHelpAboutApp);
 
@@ -1141,13 +1161,7 @@ void MixxxApp::slotHelpAbout()
 {
 
     DlgAbout *about = new DlgAbout(this);
-#if defined(AMD64) || defined(EM64T) || defined(x86_64)
-    about->version_label->setText(VERSION " x64");
-#elif defined(IA64)
-    about->version_label->setText(VERSION " IA64");
-#else
     about->version_label->setText(VERSION);
-#endif
     QString credits =
     QString("<p align=\"center\"><b>Mixxx %1 Development Team</b></p>"
 "<p align=\"center\">"
@@ -1198,6 +1212,8 @@ void MixxxApp::slotHelpAbout()
 "Irwin C&eacute;spedes B<br>"
 "Micz Flor<br>"
 "Daniel James<br>"
+"Mika Haulo<br>"
+"Till Hofmann<br>"
 
 "</p>"
 "<p align=\"center\"><b>And special thanks to:</b></p>"
@@ -1269,14 +1285,7 @@ void MixxxApp::slotHelpAbout()
 "Michael Pujos<br>"
 "Claudio Bantaloukas<br>"
 "Pavol Rusnak<br>"
-
-#if defined(AMD64) || defined(EM64T) || defined(x86_64)
-    "</p>").arg(VERSION " x64");
-#elif defined(IA64)
-    "</p>").arg(VERSION " IA64");
-#else
     "</p>").arg(VERSION);
-#endif
 
     about->textBrowser->setHtml(credits);
     about->show();
@@ -1290,6 +1299,12 @@ void MixxxApp::slotHelpSupport()
     QDesktopServices::openUrl(qSupportURL);
 }
 
+void MixxxApp::slotHelpFeedback() {
+    QUrl qFeedbackUrl;
+    qFeedbackUrl.setUrl(MIXXX_FEEDBACK_URL);
+    QDesktopServices::openUrl(qFeedbackUrl);
+}
+
 void MixxxApp::rebootMixxxView() {
 
     if (!m_pWidgetParent || !m_pView)
