@@ -22,6 +22,7 @@
 
 #include "skin/legacyskinparser.h"
 #include "skin/colorschemeparser.h"
+#include "skin/propertybinder.h"
 
 #include "widget/wwidget.h"
 #include "widget/wabstractcontrol.h"
@@ -135,8 +136,11 @@ QList<QString> LegacySkinParser::getSchemeList(QString qSkinPath) {
 // static
 void LegacySkinParser::freeChannelStrings() {
     QMutexLocker lock(&s_safeStringMutex);
-    foreach (const char *s, s_channelStrs) {
-        free((void*) s); // created using strdup/malloc so use free
+    for (int i = 0; i < s_channelStrs.length(); ++i) {
+        if (s_channelStrs[i]) {
+            delete [] s_channelStrs[i];
+        }
+        s_channelStrs[i] = NULL;
     }
 }
 
@@ -209,12 +213,22 @@ QWidget* LegacySkinParser::parseNode(QDomElement node, QWidget *pGrandparent) {
     QString nodeName = node.nodeName();
     //qDebug() << "parseNode" << node.nodeName();
 
-    if (nodeName == "skin") {
-        // Root of the document
-    }
-
     // TODO(rryan) replace with a map to function pointers?
-    if (nodeName == "Background") {
+
+    // Root of the document
+    if (nodeName == "skin") {
+        // Descend chilren, should only happen for the root node
+        QDomNodeList children = node.childNodes();
+
+        for (int i = 0; i < children.count(); ++i) {
+            QDomNode node = children.at(i);
+
+            if (node.isElement()) {
+                parseNode(node.toElement(), pGrandparent);
+            }
+        }
+        return pGrandparent;
+    } else if (nodeName == "Background") {
         return parseBackground(node, pGrandparent);
     } else if (nodeName == "SliderComposed") {
         return parseSliderComposed(node);
@@ -252,37 +266,40 @@ QWidget* LegacySkinParser::parseNode(QDomElement node, QWidget *pGrandparent) {
         return parseWidgetGroup(node);
     } else if (nodeName == "Style") {
         return parseStyle(node);
+    } else {
+        qDebug() << "Invalid node name in skin:" << nodeName;
     }
 
-    // Descend chilren, should only happen for the root node
-    QDomNodeList children = node.childNodes();
-
-    for (int i = 0; i < children.count(); ++i) {
-        QDomNode node = children.at(i);
-
-        if (node.isElement()) {
-            parseNode(node.toElement(), pGrandparent);
-        }
-    }
-
-    return pGrandparent;
+    return NULL;
 }
 
 QWidget* LegacySkinParser::parseWidgetGroup(QDomElement node) {
     QWidget* pGroup = new QGroupBox(m_pParent);
-
+    pGroup->setContentsMargins(0, 0, 0, 0);
     setupWidget(node, pGroup);
+    setupConnections(node, pGroup);
 
+    QBoxLayout* pLayout = NULL;
     if (!XmlParse::selectNode(node, "Layout").isNull()) {
         QString layout = XmlParse::selectNodeQString(node, "Layout");
         if (layout == "vertical") {
-
+            pLayout = new QVBoxLayout();
+            pLayout->setSpacing(0);
+            pLayout->setContentsMargins(0, 0, 0, 0);
+            pLayout->setAlignment(Qt::AlignCenter);
         } else if (layout == "horizontal") {
-
+            pLayout = new QHBoxLayout();
+            pLayout->setSpacing(0);
+            pLayout->setContentsMargins(0, 0, 0, 0);
+            pLayout->setAlignment(Qt::AlignCenter);
         }
     }
 
     QDomNode childrenNode = XmlParse::selectNode(node, "Children");
+
+    QWidget* pOldParent = m_pParent;
+    m_pParent = pGroup;
+
     if (!childrenNode.isNull()) {
         // Descend chilren
         QDomNodeList children = childrenNode.childNodes();
@@ -291,10 +308,22 @@ QWidget* LegacySkinParser::parseWidgetGroup(QDomElement node) {
             QDomNode node = children.at(i);
 
             if (node.isElement()) {
-                parseNode(node.toElement(), pGroup);
+                QWidget* pChild = parseNode(node.toElement(), pGroup);
+
+                if (pChild == NULL)
+                    continue;
+
+                if (pLayout) {
+                    pLayout->addWidget(pChild);
+                }
             }
         }
     }
+    m_pParent = pOldParent;
+
+    if (pLayout) {
+        pGroup->setLayout(pLayout);
+    }
 
     return pGroup;
 }
@@ -368,6 +397,9 @@ QWidget* LegacySkinParser::parseOverview(QDomElement node) {
 
     WOverview* p = new WOverview(pSafeChannelStr, m_pParent);
 
+    connect(p, SIGNAL(trackDropped(QString, QString)),
+            m_pPlayerManager, SLOT(slotLoadToPlayer(QString, QString)));
+
     setupWidget(node, p);
     p->setup(node);
     setupConnections(node, p);
@@ -408,7 +440,8 @@ QWidget* LegacySkinParser::parseVisual(QDomElement node) {
     ControlObjectThreadWidget * p = new ControlObjectThreadWidget(
         ControlObject::getControl(ConfigKey(channelStr, "wheel")));
 
-    p->setWidget((QWidget *)widget, true, true, true, Qt::LeftButton);
+    p->setWidget((QWidget *)widget, true, true,
+                 ControlObjectThreadWidget::EMIT_ON_PRESS, Qt::LeftButton);
 
     setupWidget(node, widget);
     if (type == WAVEFORM_GL) {
@@ -606,23 +639,8 @@ QWidget* LegacySkinParser::parseKnob(QDomElement node) {
 QWidget* LegacySkinParser::parseTableView(QDomElement node) {
     QStackedWidget* pTabWidget = new QStackedWidget(m_pParent);
 
-    // Position
-    if (!XmlParse::selectNode(node, "Pos").isNull())
-    {
-        QString pos = XmlParse::selectNodeQString(node, "Pos");
-        int x = pos.left(pos.indexOf(",")).toInt();
-        int y = pos.mid(pos.indexOf(",")+1).toInt();
-        pTabWidget->move(x,y);
-    }
-
-    // Size
-    if (!XmlParse::selectNode(node, "Size").isNull())
-    {
-        QString size = XmlParse::selectNodeQString(node, "Size");
-        int x = size.left(size.indexOf(",")).toInt();
-        int y = size.mid(size.indexOf(",")+1).toInt();
-        pTabWidget->setFixedSize(x,y);
-    }
+    setupPosition(node, pTabWidget);
+    setupSize(node, pTabWidget);
 
     QWidget* pLibraryPage = new QWidget(pTabWidget);
 
@@ -806,7 +824,9 @@ const char* LegacySkinParser::safeChannelString(QString channelStr) {
         }
     }
     QByteArray qba(channelStr.toAscii());
-    const char *safe(strdup(qba.constData()));
+    char *safe = new char[qba.size() + 1]; // +1 for \0
+    int i = 0;
+    while (safe[i] = qba[i]) ++i;
     s_channelStrs.append(safe);
     return safe;
 }
@@ -817,24 +837,74 @@ QWidget* LegacySkinParser::parseStyle(QDomElement node) {
     return m_pParent;
 }
 
-void LegacySkinParser::setupWidget(QDomNode node, QWidget* pWidget) {
-    // Position
-    if (!XmlParse::selectNode(node, "Pos").isNull())
-    {
+void LegacySkinParser::setupPosition(QDomNode node, QWidget* pWidget) {
+    if (!XmlParse::selectNode(node, "Pos").isNull()) {
         QString pos = XmlParse::selectNodeQString(node, "Pos");
         int x = pos.left(pos.indexOf(",")).toInt();
         int y = pos.mid(pos.indexOf(",")+1).toInt();
         pWidget->move(x,y);
     }
+}
 
-    // Size
-    if (!XmlParse::selectNode(node, "Size").isNull())
-    {
+void LegacySkinParser::setupSize(QDomNode node, QWidget* pWidget) {
+    if (!XmlParse::selectNode(node, "Size").isNull()) {
         QString size = XmlParse::selectNodeQString(node, "Size");
-        int x = size.left(size.indexOf(",")).toInt();
-        int y = size.mid(size.indexOf(",")+1).toInt();
-        pWidget->setFixedSize(x,y);
+        int comma = size.indexOf(",");
+        QString xs = size.left(comma);
+        QString ys = size.mid(comma+1);
+        QSizePolicy sizePolicy;
+
+        if (xs.endsWith("e")) {
+            //qDebug() << "horizontal expanding";
+            sizePolicy.setHorizontalPolicy(QSizePolicy::Expanding);
+            xs = xs.left(xs.size()-1);
+        } else if (xs.endsWith("me")) {
+            //qDebug() << "horizontal minimum expanding";
+            sizePolicy.setHorizontalPolicy(QSizePolicy::MinimumExpanding);
+            xs = xs.left(xs.size()-2);
+        } else if (xs.endsWith("i")) {
+            //qDebug() << "horizontal ignored";
+            sizePolicy.setHorizontalPolicy(QSizePolicy::Ignored);
+            xs = xs.left(xs.size()-1);
+        } else {
+            sizePolicy.setHorizontalPolicy(QSizePolicy::Fixed);
+        }
+
+        bool ok;
+        int x = xs.toInt(&ok);
+        if (ok) {
+            //qDebug() << "setting width to" << x;
+            pWidget->setMinimumWidth(x);
+        }
+
+        if (ys.endsWith("e")) {
+            //qDebug() << "vertical expanding";
+            sizePolicy.setVerticalPolicy(QSizePolicy::Expanding);
+            ys = ys.left(ys.size()-1);
+        } else if (ys.endsWith("me")) {
+            //qDebug() << "vertical minimum expanding";
+            sizePolicy.setVerticalPolicy(QSizePolicy::MinimumExpanding);
+            ys = ys.left(ys.size()-2);
+        } else if (ys.endsWith("i")) {
+            //qDebug() << "vertical ignored";
+            sizePolicy.setVerticalPolicy(QSizePolicy::Ignored);
+            ys = ys.left(ys.size()-1);
+        } else {
+            sizePolicy.setVerticalPolicy(QSizePolicy::Fixed);
+        }
+
+        int y = ys.toInt(&ok);
+        if (ok) {
+            //qDebug() << "setting height to" << y;
+            pWidget->setMinimumHeight(y);
+        }
+        pWidget->setSizePolicy(sizePolicy);
     }
+}
+
+void LegacySkinParser::setupWidget(QDomNode node, QWidget* pWidget) {
+    setupPosition(node, pWidget);
+    setupSize(node, pWidget);
 
     // Tooltip
     if (!XmlParse::selectNode(node, "Tooltip").isNull()) {
@@ -843,9 +913,9 @@ void LegacySkinParser::setupWidget(QDomNode node, QWidget* pWidget) {
     }
 
     QString style = XmlParse::selectNodeQString(node, "Style");
-    if (style != "")
+    if (style != "") {
         pWidget->setStyleSheet(style);
-
+    }
 }
 
 void LegacySkinParser::setupConnections(QDomNode node, QWidget* pWidget) {
@@ -863,23 +933,30 @@ void LegacySkinParser::setupConnections(QDomNode node, QWidget* pWidget) {
         ControlObject * control = ControlObject::getControl(configKey);
 
         if (control == NULL) {
-            qWarning() << "Requested control does not exist:" << key;
-            con = con.nextSibling();
-            continue;
+            qWarning() << "Requested control does not exist:" << key << ". Creating it.";
+            control = new ControlObject(configKey);
         }
 
-        if (!XmlParse::selectNode(con, "OnOff").isNull() &&
-            XmlParse::selectNodeQString(con, "OnOff")=="true")
-        {
+        QString property = XmlParse::selectNodeQString(con, "BindProperty");
+        if (property != "") {
+            qDebug() << "Making property binder for" << property;
+            // Bind this control to a property. Not leaked because it is
+            // parented to the widget and so it dies with it.
+            new PropertyBinder(pWidget, property, control);
+        } else if (!XmlParse::selectNode(con, "OnOff").isNull() &&
+            XmlParse::selectNodeQString(con, "OnOff")=="true") {
             // Connect control proxy to widget
             (new ControlObjectThreadWidget(control))->setWidgetOnOff(pWidget);
-        }
-        else
-        {
+        } else {
+            // Default to emit on press
+            ControlObjectThreadWidget::EmitOption emitOption = ControlObjectThreadWidget::EMIT_ON_PRESS;
+
             // Get properties from XML, or use defaults
-            bool bEmitOnDownPress = true;
+            if (XmlParse::selectNodeQString(con, "EmitOnPressAndRelease").contains("true", Qt::CaseInsensitive))
+                emitOption = ControlObjectThreadWidget::EMIT_ON_PRESS_AND_RELEASE;
+
             if (XmlParse::selectNodeQString(con, "EmitOnDownPress").contains("false", Qt::CaseInsensitive))
-                bEmitOnDownPress = false;
+                emitOption = ControlObjectThreadWidget::EMIT_ON_RELEASE;
 
             bool connectValueFromWidget = true;
             if (XmlParse::selectNodeQString(con, "ConnectValueFromWidget").contains("false", Qt::CaseInsensitive))
@@ -901,7 +978,7 @@ void LegacySkinParser::setupConnections(QDomNode node, QWidget* pWidget) {
             // Connect control proxy to widget
             (new ControlObjectThreadWidget(control))->setWidget(
                 pWidget, connectValueFromWidget, connectValueToWidget,
-                bEmitOnDownPress, state);
+                emitOption, state);
 
             // Add keyboard shortcut info to tooltip string
             QString tooltip = pWidget->toolTip();
