@@ -24,6 +24,7 @@
 #include "sounddevice.h"
 #include "sounddeviceportaudio.h"
 #include "audiopath.h"
+#include "controlobject.h"
 
 SoundDevicePortAudio::SoundDevicePortAudio(ConfigObject<ConfigValue> *config, SoundManager *sm,
                                            const PaDeviceInfo *deviceInfo, unsigned int devIndex)
@@ -102,12 +103,12 @@ int SoundDevicePortAudio::open()
     if (m_dSampleRate <= 0) {
         m_dSampleRate = 44100.0f;
     }
-    qDebug() << "m_dSampleRate" << m_dSampleRate;
+    qDebug() << "Requested sample rate:" << m_dSampleRate;
 
     //Get latency in milleseconds
     qDebug() << "framesPerBuffer:" << m_framesPerBuffer;
     double latencyMSec = m_framesPerBuffer / m_dSampleRate * 1000;
-    qDebug() << "latency in milliseconds:" << latencyMSec;
+    qDebug() << "Mixxx latency in milliseconds:" << latencyMSec;
 
     qDebug() << "output channels:" << m_outputParams.channelCount << "| input channels:"
         << m_inputParams.channelCount;
@@ -210,6 +211,12 @@ int SoundDevicePortAudio::open()
     }
     else
         qDebug() << "PortAudio: Started stream successfully";
+    
+    // Get the actual details of the stream & update Mixxx's data
+    const PaStreamInfo* streamDetails = Pa_GetStreamInfo(m_pStream);
+    m_dSampleRate = streamDetails->sampleRate;
+    latencyMSec = streamDetails->outputLatency*1000;
+    qDebug() << "Actual sample rate: " << m_dSampleRate << "Hz, latency:" << latencyMSec << "ms";
 
     //Update the samplerate and latency ControlObjects, which allow the waveform view to properly correct
     //for the latency.
@@ -334,7 +341,7 @@ int SoundDevicePortAudio::callbackProcess(unsigned long framesPerBuffer, float *
     {
         assert(iFrameSize > 0);
         QHash<AudioOutput, const CSAMPLE*> outputAudio
-            = m_pSoundManager->requestBuffer(m_audioOutputs, framesPerBuffer);
+            = m_pSoundManager->requestBuffer(m_audioOutputs, framesPerBuffer, this, Pa_GetStreamTime(m_pStream));
 
         // Reset sample for each open channel
         memset(output, 0, framesPerBuffer * iFrameSize * sizeof(*output));
