@@ -1,10 +1,9 @@
-/***************************************************************************
-                          soundmanager.cpp
-                             -------------------
-    begin                : Sun Aug 15, 2007
-    copyright            : (C) 2007 Albert Santoni
-    email                : gamegod \a\t users.sf.net
-***************************************************************************/
+/**
+ * @file soundmanager.cpp
+ * @author Albert Santoni <gamegod at users dot sf dot net>
+ * @author Bill Good <bkgood at gmail dot com>
+ * @date 20070815
+ */
 
 /***************************************************************************
 *                                                                         *
@@ -17,35 +16,42 @@
 
 #include <QtDebug>
 #include <QtCore>
-#include <portaudio.h>
 #include <cstring> // for memcpy and strcmp
+
+#ifdef __PORTAUDIO__
+#include <portaudio.h>
+#endif // ifdef __PORTAUDIO__
+
 #include "soundmanager.h"
 #include "sounddevice.h"
 #include "sounddeviceportaudio.h"
 #include "engine/enginemaster.h"
 #include "controlobjectthreadmain.h"
 #include "audiopath.h"
+#include "controlobject.h"
+#include "vinylcontrolproxy.h"
+
+#ifdef __PORTAUDIO__
+typedef PaError (*SetJackClientName)(const char *name);
+#endif
 
 /** Initializes Mixxx's audio core
  *  @param pConfig The config key table
- *  @param _master A pointer to the audio engine's mastering class.
+ *  @param pMaster A pointer to the audio engine's mastering class.
  */
-SoundManager::SoundManager(ConfigObject<ConfigValue> * pConfig, EngineMaster * _master)
+SoundManager::SoundManager(ConfigObject<ConfigValue> *pConfig, EngineMaster *pMaster)
     : QObject()
+    , m_pMaster(pMaster)
+    , m_pConfig(pConfig)
+    , m_pClkRefDevice(NULL)
+    , m_outputDevicesOpened(0)
+    , m_inputDevicesOpened(0)
     , m_pErrorDevice(NULL)
 #ifdef __PORTAUDIO__
     , m_paInitialized(false)
     , m_jackSampleRate(-1)
 #endif
 {
-    //qDebug() << "SoundManager::SoundManager()";
-    m_pConfig = pConfig;
-    m_pMaster = _master;
-
-    iNumDevicesOpenedForOutput = 0;
-    iNumDevicesOpenedForInput = 0;
-    iNumDevicesHaveRequestedBuffer = 0;
-
     //These are ControlObjectThreadMains because all the code that
     //uses them is called from the GUI thread (stuff like opening soundcards).
     ControlObjectThreadMain* pControlObjectLatency = new ControlObjectThreadMain(ControlObject::getControl(ConfigKey("[Master]", "latency")));
@@ -99,6 +105,17 @@ SoundManager::~SoundManager()
     // by clearDeviceList -- bkgood
 }
 
+/**
+ * Clears all variables used in operation.
+ * @note This is in a function because it's done in a few places
+ */
+void SoundManager::clearOperativeVariables()
+{
+    m_outputDevicesOpened = 0;
+    m_inputDevicesOpened = 0;
+    m_pClkRefDevice = NULL;
+}
+
 /**
  * Returns a pointer to the EngineMaster instance this SoundManager
  * is using.
@@ -208,9 +225,7 @@ void SoundManager::closeDevices()
     //requestBufferMutex.unlock();
 
     //requestBufferMutex.lock();
-    iNumDevicesOpenedForOutput = 0;
-    iNumDevicesOpenedForInput = 0;
-    iNumDevicesHaveRequestedBuffer = 0;
+    clearOperativeVariables();
     //requestBufferMutex.unlock();
 
     m_outputBuffers.clear(); // anti-cruft (safe because outputs only have
@@ -229,8 +244,8 @@ void SoundManager::closeDevices()
     // TODO(bkgood) see comment where these objects are created in setupDevices,
     // this should probably be in the dtor or at least somewhere other
     // than here.
-    while (!m_VinylControl.empty()) {
-        VinylControlProxy *vc = m_VinylControl.takeLast();
+    while (!m_vinylControl.empty()) {
+        VinylControlProxy *vc = m_vinylControl.takeLast();
         if (vc != NULL) {
             delete vc;
         }
@@ -297,6 +312,9 @@ void SoundManager::queryDevices()
 #ifdef __PORTAUDIO__
     PaError err = paNoError;
     if (!m_paInitialized) {
+#ifdef Q_OS_LINUX
+        setJACKName();
+#endif
         err = Pa_Initialize();
         m_paInitialized = true;
     }
@@ -351,7 +369,7 @@ int SoundManager::setupDevices()
 {
     qDebug() << "SoundManager::setupDevices()";
     int err = 0;
-    iNumDevicesOpenedForOutput = iNumDevicesOpenedForInput = 0;
+    clearOperativeVariables();
     int devicesAttempted = 0;
     int devicesOpened = 0;
 
@@ -366,8 +384,8 @@ int SoundManager::setupDevices()
     // TODO(bkgood) this ought to be done in the ctor or something. Not here. Really
     // shouldn't be any reason for these to be reinitialized every time the
     // audio prefs are updated. Will require work in DlgPrefVinyl.
-    m_VinylControl.append(new VinylControlProxy(m_pConfig, "[Channel1]"));
-    m_VinylControl.append(new VinylControlProxy(m_pConfig, "[Channel2]"));
+    m_vinylControl.append(new VinylControlProxy(m_pConfig, "[Channel1]"));
+    m_vinylControl.append(new VinylControlProxy(m_pConfig, "[Channel2]"));
 #endif
     foreach (SoundDevice *device, m_devices) {
         bool isInput = false;
@@ -402,12 +420,19 @@ int SoundManager::setupDevices()
             switch (out.getType()) {
             case AudioPath::MASTER:
                 m_outputBuffers[out] = m_pMaster->getMasterBuffer();
+                // Set the master output device's clock as the reference
+                //  to avoid any clock-related disturbances to the public address
+                m_pClkRefDevice = device;
                 break;
             case AudioPath::HEADPHONES:
                 m_outputBuffers[out] = m_pMaster->getHeadphoneBuffer();
                 break;
             case AudioPath::DECK:
                 m_outputBuffers[out] = m_pMaster->getChannelBuffer(out.getIndex());
+                // If a reference device has not yet been set (such as when the Master output
+                //  isn't being used with an external mixer,) use the first deck output
+                //  to avoid any clock-related disturbances on at least one public output
+                if (!m_pClkRefDevice) m_pClkRefDevice = device;
                 break;
             default:
                 break;
@@ -423,15 +448,29 @@ int SoundManager::setupDevices()
             } else {
                 ++devicesOpened;
                 if (isOutput)
-                    ++iNumDevicesOpenedForOutput;
+                    ++m_outputDevicesOpened;
                 if (isInput)
-                    ++iNumDevicesOpenedForInput;
+                    ++m_inputDevicesOpened;
             }
         }
     }
 
-    qDebug() << "iNumDevicesOpenedForOutput:" << iNumDevicesOpenedForOutput;
-    qDebug() << "iNumDevicesOpenedForInput:" << iNumDevicesOpenedForInput;
+    if (!m_pClkRefDevice && m_outputDevicesOpened > 0) {
+        QList<SoundDevice*> outputDevices = getDeviceList(m_config.getAPI(), true, false);
+        Q_ASSERT(outputDevices.length());
+        SoundDevice* device = outputDevices.first();
+        qWarning() << "Output sound device clock reference not set! Using"
+            << device->getDisplayName();
+        m_pClkRefDevice = device;
+    } else if (m_outputDevicesOpened > 0) {
+        qDebug() << "Using" << m_pClkRefDevice->getDisplayName()
+            << "as output sound device clock reference";
+    } else {
+        qDebug() << "No output devices opened, no clock reference device set";
+    }
+
+    qDebug() << m_outputDevicesOpened << "output sound devices opened";
+    qDebug() << m_inputDevicesOpened << "input sound devices opened";
 
     // returns OK if we were able to open all the devices the user
     // wanted
@@ -485,33 +524,55 @@ void SoundManager::sync()
 
 //Requests a buffer in the proper format, if we're prepared to give one.
 QHash<AudioOutput, const CSAMPLE*>
-SoundManager::requestBuffer(QList<AudioOutput> outputs, unsigned long iFramesPerBuffer)
+SoundManager::requestBuffer(QList<AudioOutput> outputs,
+    unsigned long iFramesPerBuffer, SoundDevice* device,
+    double streamTime /* = 0 */)
 {
     Q_UNUSED(outputs); // unused, we just give the caller the full hash -bkgood
     //qDebug() << "SoundManager::requestBuffer()";
 
-    //qDebug() << "numOpenedDevices" << iNumOpenedDevices;
-    //qDebug() << "iNumDevicesHaveRequestedBuffer" << iNumDevicesHaveRequestedBuffer;
-
-    //When the first device requests a buffer...
-    if (requestBufferMutex.tryLock())
-    {
-        if (iNumDevicesHaveRequestedBuffer == 0)
-        {
-            //First, sync control parameters with changes from GUI thread
-            sync();
-
-            //Process a block of samples for output. iFramesPerBuffer is the
-            //number of samples for one channel, but the EngineObject
-            //architecture expects number of samples for two channels
-            //as input (buffer size) so...
-            m_pMaster->process(0, 0, iFramesPerBuffer*2);
+    /*
+    // Display when sound cards drop or duplicate buffers (use for testing only)
+    if (iNumDevicesOpenedForOutput>1) {
+        // Running total of requested frames
+        long currentFrameCount = 0;
+        if (m_deviceFrameCount.contains(device)) currentFrameCount=m_deviceFrameCount.value(device);
+        m_deviceFrameCount.insert(device, currentFrameCount+iFramesPerBuffer);  // Overwrites existing value if already present
+        // Get current time in milliseconds
+//         uint t = QDateTime::currentDateTime().toTime_t()*1000+QDateTime::currentDateTime().toString("zzz").toUint();
 
+        if (device != m_pClkRefDevice) {  // If not the reference device,
+            // Detect dropped frames/buffers
+            long sdifference = m_deviceFrameCount.value(m_pClkRefDevice)-m_deviceFrameCount.value(device);
+            QString message = "dropped";
+            if (sdifference < 0) message = "duplicated";
+            if (sdifference != 0) {
+                m_deviceFrameCount.clear();
+                message = QString("%1 %2 %3 frames (%4 buffers)")
+                            .arg(device->getDisplayName())
+                            .arg(message)
+                            .arg(fabs(sdifference))
+                            .arg(fabs(sdifference)/iFramesPerBuffer);
+                qWarning() << message;
+            }
         }
-        iNumDevicesHaveRequestedBuffer++;
+    }
+    //  End dropped/duped buffer display
+    */
 
-        if (iNumDevicesHaveRequestedBuffer >= iNumDevicesOpenedForOutput)
-            iNumDevicesHaveRequestedBuffer = 0;
+    //When the clock reference device requests a buffer...
+    if (device == m_pClkRefDevice && requestBufferMutex.tryLock())
+    {
+        // Only generate a new buffer for the clock reference card
+//         qDebug() << "New buffer for" << device->getDisplayName() << "of size" << iFramesPerBuffer;
+        //First, sync control parameters with changes from GUI thread
+        sync();
+
+        //Process a block of samples for output. iFramesPerBuffer is the
+        //number of samples for one channel, but the EngineObject
+        //architecture expects number of samples for two channels
+        //as input (buffer size) so...
+        m_pMaster->process(0, 0, iFramesPerBuffer*2);
 
         requestBufferMutex.unlock();
     }
@@ -610,8 +671,8 @@ void SoundManager::pushBuffer(QList<AudioInput> inputs, short * inputBuffer,
             if (in.getType() == AudioInput::VINYLCONTROL) {
                 unsigned int index = in.getIndex();
                 Q_ASSERT(index < 2); // XXX we only do two vc decks atm -- bkgood
-                if (m_VinylControl[index] && m_inputBuffers.contains(in)) {
-                    m_VinylControl[index]->AnalyseSamples(m_inputBuffers[in], iFramesPerBuffer);
+                if (m_vinylControl[index] && m_inputBuffers.contains(in)) {
+                    m_vinylControl[index]->AnalyseSamples(m_inputBuffers[in], iFramesPerBuffer);
                 }
             }
         }
@@ -620,3 +681,25 @@ void SoundManager::pushBuffer(QList<AudioInput> inputs, short * inputBuffer,
     //TODO: Add pass-through option here (and push it into EngineMaster)...
     //      (or maybe save it, and then have requestBuffer() push it into EngineMaster)...
 }
+
+void SoundManager::setJACKName() const
+{
+#ifdef __PORTAUDIO__
+#ifdef Q_OS_LINUX
+    typedef PaError (*SetJackClientName)(const char *name);
+    QLibrary portaudio("libportaudio.so.2");
+    if (portaudio.load()) {
+        SetJackClientName func(
+            reinterpret_cast<SetJackClientName>(
+                portaudio.resolve("PaJack_SetClientName")));
+        if (func) {
+            if (!func("Mixxx")) qDebug() << "JACK client name set";
+        } else {
+            qWarning() << "failed to resolve JACK name method";
+        }
+    } else {
+        qWarning() << "failed to load portaudio for JACK rename";
+    }
+#endif
+#endif
+}
